% Feb 20th 2020.
%Authors:  Eric J. Goldfarb, Ken Ikeda, Richard A. Ketcham, Maa Prodanovic and Nicola Tisato

clc
close all 
clear all


% This is to create a new.mat file from CT tiff images
%replace "...." where your folder is with your tiff files. 
a=dir('....\*.tif');
for n=1:length(a)
    b = imread([('....\') a(n).name]);
    CT(:,:,n)=double(b);
end


%for this dataset, we have pre located where your target locations are. 
%using the function file "SelectData", you can automatically select the ideal
%location in the dataset for the target. Uncomment the target choices for
%the sample you choose to run

% % %Target Locations for Sample 1
% % CT_Dist.air=SelectData(CT,[775:800],[0,14],[130,272]);
% % CT_Dist.borosillicate=SelectData(CT,[865:880],[0, 11],[211,246]);
% % CT_Dist.glass=SelectData(CT,[690:718],[0, 12],[140,269]);;
% % CT_Dist.quartz=SelectData(CT,[890:905],[0, 12],[140,269]);
%  %here is our data cropping window/subcube to use.
% % crop=CT(150:299,150:299,175:574);
% % 
% % %Target Locations for Sample 2
% % CT_Dist.air=SelectData(CT,[775:800],[0,15],[130,255]);
% % CT_Dist.borosillicate=SelectData(CT,[885:905],[0, 15],[215,260]);;
% % CT_Dist.glass=SelectData(CT,[695:730],[0,25],[160,210]);;
% % CT_Dist.quartz=SelectData(CT,[895:915],[0, 13],[143,254]);
% % crop=CT(150:299,150:299,175:574);
% % 
% % %Target Locations for Sample 3
% % CT_Dist.air=SelectData(CT,[772:800],[0,15],[240,283]);
% % CT_Dist.borosillicate=SelectData(CT,[845:860],[0,15],[274,211]);
% % CT_Dist.glass=SelectData(CT,[650:680],[0,50],[236,240]);
% % CT_Dist.quartz=SelectData(CT,[850:865],[0,12],[237,281]);
% % crop=CT(180:329,180:329,175:574);

% % %Target Locations for Sample 4
% % CT_Dist.air=SelectData(CT,[815:835],[0,15],[227,168])+5; %bottom of scan is clipped and mean air is 5 too low from visual pic on the histogram
% % CT_Dist.borosillicate=SelectData(CT,[900:920],[0, 10],[139,173]);;
% % CT_Dist.glass=SelectData(CT,[720:760],[0,15],[160,210]);
% % CT_Dist.quartz=SelectData(CT,[910:930],[0, 16],[215,172]);
% % crop=CT(150:299,150:299,175:574);
% % 



%Here are the estimates for physical density. We are most confident the
%middle value is accurate. The first and lest value is a lower level, and
%upper level range of value respectively. 
densitiesWithError.air = [1.2041 1.1839 1.225];   
densitiesWithError.borosillicate = [ 2212.3 2214  2215.7];
densitiesWithError.glass = [ 2504.1    2508.1    2512.1];
densitiesWithError.quartz = [2650 2655 2660];      
%use the following funciton to create a fitting curve that incorporates
%error in X and Y.
[equation,sigma_intercept, sigma_slope]=DeriveNonSegCurveWithError(CT_Dist,densitiesWithError)


CT_size=size(crop);

%normalise all the points by removing the value of air from the entire
%dataset. Then, plug it into the equation
Density_rock=(crop-nanmean(CT_Dist.air(:)));
Density_rock=equation(Density_rock(:));
Density_rock=reshape(Density_rock,(CT_size));

%set the density of the most dense mineral phase, and estimate porosity
%with the following function
upper_limit=sum(2655);
[Porosity_rock,NonSegporosity]=Density2Porosity(Density_rock, upper_limit);

%calculate mean density
mean_density=nanmean(Density_rock(:))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% Uncertainty %%%%%%%%%%%%%%%%%%%%%%%


%declare error for intercept and slope 
s_a = sigma_intercept;
s_b = sigma_slope;

error_frame = 5 ;%this is the range from the quartz density inputs
error_fluid = 0.0206; %this is the range  from the air density inputs

d_eff = mean_density;
d_fluid = densitiesWithError.air(2) ;%density of air
d_solid = densitiesWithError.quartz(2); %density of qz

%see error derivation equations. This is setting up the variables and
%solving them 

N = length(crop(:));%Total number of voxels
sumVoxels = sum(crop(:));

uncertainty_density = sqrt(s_a^2 + (sumVoxels/N)^2*(s_b)^2) %error for density

firstTerm = (error_frame)^2 * ((d_solid - d_fluid) / (d_solid - d_fluid)^2)^2;
SecondTerm = (uncertainty_density)^2*abs(1/(d_solid - d_fluid))^2;
ThirdTerm = error_fluid^2 * ((d_solid - d_eff) / (d_solid - d_fluid)^2)^2;
uncertainty_porosity = sqrt(firstTerm + SecondTerm + ThirdTerm)  %error for porosity 
