% Feb 20th 2020.
%Authors:  Eric J. Goldfarb, Ken Ikeda, Richard A. Ketcham, Maa Prodanovi? and Nicola Tisato



function [equation,sigma_intercept, sigma_slope,Rsquared] = DeriveNonSegCurveWithError(CT_Dist,densitiesWithError)
%inputs are ct distributions for targets, and densities with error ranges.

%make sure datasets are the same size
if length(fieldnames(CT_Dist)) ~= length(fieldnames(densitiesWithError))
        sprintf( ' error, CT distributions and densities are not equally paired')
    return
end


fns = fieldnames(CT_Dist); %define the elements in CT dist


for i=1: length(fns) %for each CT distribution

    dist=CT_Dist.(fns{i})   ;%assign to the ith row
    CT_best (i) = nanmean(dist(:));  %assign best number      
    gaussian.(fns{i})=fitdist(dist(:), 'Normal'); %find the gaussian distribution
    sigma_CT(i)=   2.*gaussian.(fns{i}).sigma     ;    %2 SD for the CT distributions 

    density_temp=densitiesWithError.(fns{i}); 
    density_best(i)=density_temp(2);
    sigma_density(i)=(densitiesWithError.(fns{i})(3) -densitiesWithError.(fns{i})(1))./2   ;%%error from the physical lab values 

    
    %Generate CT +/- 1.0 SD of each targets. The data is recorded in this
    % CT_of_a_taget = [mean - SD      mean      mean + SD]

    CT_number.(fns{i})=[nanmean(dist(:))-gaussian.(fns{i}).sigma,      CT_best(i),        nanmean(dist(:))+gaussian.(fns{i}).sigma];   %fill ct number array
end




%  best fit 
CT_best=CT_best-CT_best(1);  %air is assumed to be first!! subtract air values



%ok so this deals with a small glitch - when its maxed out at the 16 bit
%number, there is no SD. this just uses the SD of the prior number.
if sigma_CT(3) ==0
    sigma_CT(3) = sigma_CT(4)
end

if sigma_CT(4) ==0
    sigma_CT (4) ==sigma_CT(3)
end


    %The best statistical line is fit with York et al fitting technique https://www.mathworks.com/matlabcentral/fileexchange/26586-linear-regression-with-errors-in-x-and-y

 [intercept_york,slope_york, sigma_intercept, sigma_slope] = york_fit(CT_best,density_best,sigma_CT,sigma_density, zeros(1,4));


%define two points on this line so i can use a polyfit 
p1=[0, intercept_york];
p2=[10,  slope_york.*10+ intercept_york ];

[equation] = fit([p1(1) p2(1)]',[p1(2) p2(2)]' , 'poly1' );



%%%%%%%manually calculate r squared for york
Rsquared = 1 - sum((equation(CT_best)-density_best').^2)./sum((density_best - mean(density_best)).^2)


end

