% Feb 20th 2020.
%Authors:  Eric J. Goldfarb, Ken Ikeda, Richard A. Ketcham, Maa Prodanovi? and Nicola Tisato



function [data] = SelectData(data,set,radius,centre)
%inputs are: entire dataset, slices of dataset of interest, inner and outer
%radius for a cropping circle, and centre of cropping circle.
data=data(:,:,set);


gridsize=size(data);
x_size=gridsize(1);
y_size=gridsize(2);

%because the centre of the rock/area of interest is not necessarily the
%centre of the dataset, this function does not use the midpoint of
%length(x)/2 and length(y)/2. Instead, a centre value is inputted. negative
%one times the centre is the left (or top) edge. As x and y increases to
%approach the centre, the negative number goes up in the interval of one value of resolution per
%voxel, such that the negative number equals 0 by the time the centre is
%reached. After centre,the value counts up in the positive domain. In order to keep the grid size
%the same, the maximum subtracts the starting value.

x=[-1.*centre(1):x_size-((centre(1)+1))];
y=[-1.*centre(2):y_size-((centre(2)+1))];

%creates a 2D grid of x and y
[x_ref,y_ref]=meshgrid(y,x);

%distance is a 2d grid where each point is a value of distance, in metres,
%from the centre of the grid (as inputted)

distance2D=sqrt((x_ref.^2)+(y_ref.^2));
%adds a z component by using repmat.
distance_grid=repmat(distance2D,[1 1 gridsize(3)]);
data(distance_grid<radius(1) | distance_grid>radius(2))=NaN;

%Two radii are inputted at the start of the function, effectively creating
%an inner and outter circle. All numbers greater than the outer and less than the inner are
%discarded.
%  data=data(distance_grid(:,:,set)>radius(1) & distance_grid(:,:,set)<radius(2));

%  data=data(distance_grid)>radius(1) & distance_grid(:,:,set)<radius(2));


end